@@ -20,6 +20,10 @@
   Boston, MA 02110-1301, USA.
 */
 
+#ifdef HAVE_CONFIG_H
+ #include "config.h"
+#endif
+
 #include <key.h>
 
 #include "util.h"
@@ -29,6 +33,7 @@
 #include <gpgme.h>
 
 #include <string.h>
+#include <strings.h>
 #include <istream>
 #include <iterator>
 
@@ -229,6 +234,22 @@ bool Key::isQualified() const
     return key && key->is_qualified;
 }
 
+bool Key::isDeVs() const
+{
+    if (!key) {
+        return false;
+    }
+    if (!key->subkeys || !key->subkeys->is_de_vs) {
+        return false;
+    }
+    for (gpgme_sub_key_t subkey = key->subkeys ; subkey ; subkey = subkey->next) {
+        if (!subkey->is_de_vs) {
+            return false;
+        }
+    }
+    return true;
+}
+
 const char *Key::issuerSerial() const
 {
     return key ? key->issuer_serial : 0 ;
@@ -273,6 +294,7 @@ const char *Key::primaryFingerprint() const
         /* Return the first subkeys fingerprint */
         return key->subkeys->fpr;
     }
+    return nullptr;
 }
 
 unsigned int Key::keyListMode() const
@@ -335,7 +357,12 @@ void Key::update()
                         KeyListMode::Validate |
                         KeyListMode::WithTofu);
     Error err;
-    auto newKey = ctx->key(primaryFingerprint(), err, hasSecret());
+    auto newKey = ctx->key(primaryFingerprint(), err, true);
+    // Not secret so we get the information from the pubring.
+    if (newKey.isNull())
+      {
+        newKey = ctx->key(primaryFingerprint(), err, false);
+      }
     delete ctx;
     if (err) {
         return;
@@ -344,6 +371,27 @@ void Key::update()
     return;
 }
 
+// static
+Key Key::locate(const char *mbox)
+{
+    if (!mbox) {
+        return Key();
+    }
+
+    auto ctx = Context::createForProtocol(OpenPGP);
+    if (!ctx) {
+        return Key();
+    }
+
+    ctx->setKeyListMode (Extern | Local);
+
+    Error e = ctx->startKeyListing (mbox);
+    auto ret = ctx->nextKey (e);
+    delete ctx;
+
+    return ret;
+}
+
 //
 //
 // class Subkey
@@ -458,6 +506,11 @@ bool Subkey::isQualified() const
     return subkey && subkey->is_qualified;
 }
 
+bool Subkey::isDeVs() const
+{
+    return subkey && subkey->is_de_vs;
+}
+
 bool Subkey::isCardKey() const
 {
     return subkey && subkey->is_cardkey;
@@ -465,7 +518,12 @@ bool Subkey::isCardKey() const
 
 const char *Subkey::cardSerialNumber() const
 {
-    return subkey ? subkey->card_number : 0 ;
+    return subkey ? subkey->card_number : nullptr;
+}
+
+const char *Subkey::keyGrip() const
+{
+    return subkey ? subkey->keygrip : nullptr;
 }
 
 bool Subkey::isSecret() const
@@ -872,12 +930,64 @@ const char *UserID::Signature::policyURL() const
     return 0;
 }
 
+std::string UserID::addrSpecFromString(const char *userid)
+{
+    if (!userid) {
+        return std::string();
+    }
+    char *normalized = gpgme_addrspec_from_uid (userid);
+    if (normalized) {
+        std::string ret(normalized);
+        gpgme_free(normalized);
+        return ret;
+    }
+    return std::string();
+}
+
+std::string UserID::addrSpec() const
+{
+    if (!uid || !uid->address) {
+        return std::string();
+    }
+
+    return uid->address;
+}
+
+Error UserID::revoke()
+{
+    if (isNull()) {
+        return Error::fromCode(GPG_ERR_GENERAL);
+    }
+    auto ctx = Context::createForProtocol(parent().protocol());
+    if (!ctx) {
+        return Error::fromCode(GPG_ERR_INV_ENGINE);
+    }
+    Error ret = ctx->revUid(key, id());
+    delete ctx;
+    return ret;
+}
+
+Error Key::addUid(const char *uid)
+{
+    if (isNull()) {
+        return Error::fromCode(GPG_ERR_GENERAL);
+    }
+    auto ctx = Context::createForProtocol(protocol());
+    if (!ctx) {
+        return Error::fromCode(GPG_ERR_INV_ENGINE);
+    }
+    Error ret = ctx->addUid(key, uid);
+    delete ctx;
+    return ret;
+}
+
 std::ostream &operator<<(std::ostream &os, const UserID &uid)
 {
     os << "GpgME::UserID(";
     if (!uid.isNull()) {
         os << "\n name:      " << protect(uid.name())
            << "\n email:     " << protect(uid.email())
+           << "\n mbox:      " << uid.addrSpec()
            << "\n comment:   " << protect(uid.comment())
            << "\n validity:  " << uid.validityAsString()
            << "\n revoked:   " << uid.isRevoked()
